## soundcardBackend.py - Contains the backend code needed for system-config-soundcard
## Copyright (C) 2002, 2003 Red Hat, Inc.
## Copyright (C) 2002, 2003 Brent Fox <bfox@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


import string
import kudzu
import os
import getopt
import sys

##
## I18N
##
from rhpl.translate import _, N_
import rhpl.translate as translate
translate.textdomain ("system-config-soundcard")

##
## Usage
##
def usage():
    print \
_("""Usage: system-config-soundcard [OPTIONS]
Options:
  -h, --help        display this help and exit
  -k, --kernel=     which kernel version to assume:
                    2.4 (modules.conf)
                    2.6 (modprobe.conf, default)
""")

##
## Options
##

global kernel_type
kver = os.uname()[2]
kernel_type = string.join(kver.split('.')[0:2], '.')
                                                                                                              
def options():
    try:
        (opts, rest_args) = getopt.getopt(sys.argv[1:],
                                   "hk:",
                                   ["driver=", "help", "kernel="])
    except (getopt.GetoptError), e:
        print e
        print
        usage()
        sys.exit(1)
                                                                                                              
    for (opt, value) in opts:
        if opt == "-h" or opt == "--help":
            usage()
            sys.exit(0)
        if opt == "-k" or opt == "--kernel":
            kernel_type = value
            if kernel_type != '2.4' and kernel_type != '2.6':
                print "ERROR: Invalid kernel version:", kernel_type
                usage()
                sys.exit(0)

class soundcardBackend:
    def __init__(self):
        self.defaultModule = None
        self.newFile = []
        self.readFile()
        self.cardList = []
        self.dspList = []

    def probeCards(self):
        self.cardList = []
        list = kudzu.probe(kudzu.CLASS_AUDIO, kudzu.BUS_PCI, kudzu.PROBE_ALL)

        for card in list:
            #Let's go through the list of cards.  If the driver is 'unknown',
            #don't put the card in the list.
            if card.driver != "unknown":
                self.cardList.append(card)

        list = kudzu.probe(kudzu.CLASS_AUDIO, kudzu.BUS_USB, kudzu.PROBE_ALL)
        for card in list:
	    if card.driver == "snd-usb-audio":
	        self.cardList.append(card)

        return self.cardList

    def getData(self, card):
        device, module, description = card
	if (string.count(description,"|") == 1):
            maker, model = string.split(description, "|")
	else:
	    maker, model = string.split(description, " ", 1)
        return device, module, maker, model

    def playTestSound(self, module, index):
        self.setVolume(index)

        if kernel_type == '2.6':
            if self.cardList == []:
	        device = "default"
	    else:
	        device = "hw:%d,0"% index

	    path = "/usr/bin/aplay"
	    args = [path,  "-D", device, "/usr/share/system-config-soundcard/sound-sample.wav"]

        if kernel_type == "2.4":
	    if self.dspList == []:
                device = "/dev/dsp"
            else:
                device = "/dev/dsp%d" % self.dspList.index(module)

	    path = "/usr/bin/play"
	    args = [path,  "-d", device, "/usr/share/system-config-soundcard/sound-sample.wav", "-v", "2"]

        pid = os.fork()
        if (not pid):
	    os.execv(path, args)
	else:
	    os.waitpid(pid,0)

    def readFile(self):
        if kernel_type == '2.6':
            fd = open('/etc/modprobe.conf', 'r')
        if kernel_type == '2.4':
            fd = open('/etc/modules.conf', 'r')
        lines = fd.readlines()
        fd.close()

        self.newFile = []

        for line in lines:
            #strip out all lines dealing with sound
            if (string.find(line, "sound-slot-") < 0 and string.find(line,"snd-card-") < 0) or string.strip(line)[0] == "#":
                self.newFile.append(line)

            else:
                if string.find(line, "alias") > -1 and (string.find(line, "sound-slot-0") > -1 or string.find(line, "snd-card-0") > -1):
                    try:
                        a, b, self.defaultModule = string.split(line)
                    except:
                        self.defaultModule = None

    def getDefaultCard(self):
        for card in self.cardList:
            device, module, maker, model = self.getData(card)
            
            if module == self.defaultModule:
                return card

    def writeFile(self, selectedCard):
        self.selectedCard = selectedCard
        aliasList = []
        aumixList = []
        self.dspList = []

        device, self.defaultModule, maker, model = self.getData(self.cardList[selectedCard])
        if kernel_type == '2.6':
	    aliasList.append("alias snd-card-0 %s\n" % self.defaultModule)
            aumixList.append("install %s /sbin/modprobe --ignore-install %s && /usr/sbin/alsactl restore >/dev/null 2>&1 ||:\n" % (self.defaultModule, self.defaultModule)) 
            aumixList.append("remove %s { /usr/sbin/alsactl store >/dev/null 2>&1 || : ; }; /sbin/modprobe -r --ignore-remove %s\n" % (self.defaultModule, self.defaultModule))
        if kernel_type == '2.4':
	    aliasList.append("alias sound-slot-0 %s\n" % self.defaultModule)
            aumixList.append("post-install sound-slot-0 /bin/aumix-minimal -f /etc/.aumixrc -L >/dev/null 2>&1 || :\n")
            aumixList.append("pre-remove sound-slot-0 /bin/aumix-minimal -f /etc/.aumixrc -S >/dev/null 2>&1 || :\n")
        self.dspList.append(self.defaultModule)

        count = 1
        for card in self.cardList:
            device, module, maker, model = self.getData(card)

            if module == self.defaultModule:
                #We've already written this line, so skip
                pass
            else:
                if kernel_type == '2.6':
		    aliasList.append("alias snd-card-%d %s\n" % (count, module))
		    aumixList.append("install %s /sbin/modprobe --ignore-install %s && /usr/sbin/alsactl restore >/dev/null 2>&1 ||:\n" % (module, module)) 
                    aumixList.append("remove %s { /usr/sbin/alsactl store >/dev/null 2>&1 || : ; }; /sbin/modprobe -r --ignore-remove %s\n" % (module, module))
                if kernel_type == '2.4':
		    aliasList.append("alias sound-slot-%d %s\n" % (count, module))
                    aumixList.append("post-install sound-slot-%d /bin/aumix-minimal -f /etc/.aumixrc -L >/dev/null 2>&1 || :\n" % count)
                    aumixList.append("pre-remove sound-slot-%d /bin/aumix-minimal -f /etc/.aumixrc -S >/dev/null 2>&1 || :\n" % count)
                self.dspList.append(module)
                count = count + 1

        #Let's re-read the file again to make sure we're starting from a clean base
        self.readFile()
        self.newFile = self.newFile + aliasList + aumixList

        if kernel_type == '2.6':
            fd = open('/etc/modprobe.conf', 'w')
        if kernel_type == '2.4':
            fd = open('/etc/modules.conf', 'w')

        for line in self.newFile:
            fd.write(line)

        fd.close()

    def unloadModules(self):
        for card in self.cardList:
            device, module, maker, model = self.getData(card)

            retval = os.system("/sbin/modprobe -r %s" % module)

    def setVolume(self, index):
        #alsa drivers load muted. we need to set the volume for them.
        if kernel_type == '2.4':
            aumix_path = "/bin/aumix-minimal"
            aumix_pid = os.fork()
            aumix_args = [aumix_path, "-f", "/etc/.aumixrc", "-L"]

            if (not aumix_pid):
		try:
		    os.execv(aumix_path, aumix_args)
		except:
		    sys.exit(0)
	    else:
		os.waitpid(aumix_pid,0)

        if kernel_type == '2.6':
            amixer_path = "/bin/alsaunmute"
            amixer_pid = os.fork()
            amixer_args = [amixer_path, `index`, "-v" ]
            
            if (not amixer_pid):
		try:
		    os.execv(amixer_path, amixer_args)
		except:
		    sys.exit(0)
	    else:
		os.waitpid(amixer_pid,0)

    def setDefault(self, index):
        if kernel_type == '2.6':
	     lines = []
	     lines.append("# Generated by system-config-soundcard, do not edit by hand\n")
	     lines.append("pcm.!default { type hw card %d }\n" % index)
	     lines.append("ctl.!default { type hw card %d }\n" % index)

	     fd = open('/etc/asound.conf', 'w')
	     for line in lines:
	         fd.write(line)
	     fd.close()

    def loadModules(self):
        retval = os.system("/sbin/modprobe %s" % self.defaultModule)
        if retval == 0:
            #Modprobe worked
            pass
        else:
            #Modprobe failed
            return self.defaultModule

        for card in self.cardList:
            device, module, maker, model = self.getData(card)

            if module == self.defaultModule:
                #We've already inserted this one, so don't try again
                pass
            else:
                retval = os.system("/sbin/modprobe %s" % module)

                if retval == 0:
                    #Modprobe worked
                    pass
                else:
                    #Modprobe failed
                    return self.defaultModule

        return 0

